/*-----------------------------------------------------------------------------
 * File: SerPortConfig.java
 * Function: Serial port configuration class. This object is passed to
 *           the SerialPort object when opening or configuring.
 *
 *---------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
 * Solutions Consulting makes no representations or warranties about the
 * suitability of the software, either express or implied, including but
 * not limited to the implied warranties of merchantability, fitness for
 * a particular purpose, or non-infringement. Solutions Consulting shall
 * not be liable for any damages suffered by licensee as a result of
 * using, modifying or distributing this software or its derivatives.
 *---------------------------------------------------------------------------*/
//package SerialIO;

import java.io.*;

/*---------------------------------------------------------------------------*/
/**
 * Serial port configuration class. A SerialConfig object is passed
 * to a SerialPort class when the SerialPort class in created.
 * <p><b>
 * <li>Copyright (c) 1996,1997 Solutions Consulting, All Rights Reserved.
 * </p></b>
 * @version 1.8 12 Jun 1997
 * @author David H. Boydston
 */
/*---------------------------------------------------------------------------*/
public class SerialConfig {
/**
 * Create a SerialConfig object based on 'standard' port parameters
 * @param serPort The serial port number e.g. use 1 for COM1
 */
	public SerialConfig(int serPort) {
		--serPort; // use zero based below
		port   = serPort;
		}

/**
 * Create a SerialConfig object based on 'standard' port parameters
 * @param serPort Serial port number e.g. use 1 for COM1
 * @param txBufLen Transmit buffer length (in bytes) for the port
 * @param rxBufLen Receive buffer length (in bytes) for the port
 */
	public SerialConfig(int serPort, int txBufLen, int rxBufLen)
		{
		--serPort; // use zero based below
		port   = serPort;
		txLen  = txBufLen;
		rxLen  = rxBufLen;
		}
/*
 * class method(s)
 */
/**
 * Set the bit rate
 * @exception Exception If bit rate is out of range.
 */
	public void setBitRate(int newBitRate) throws Exception {
		if (newBitRate < 0 || newBitRate > 11)
			throw new Exception("Invalid Serial Port Bit Rate");
		bitRate = newBitRate;
		}
/**
 * Set the parity
 * @exception Exception If parity is out of range.
 */
 	public void setParity(int newParity) throws Exception {
		if (newParity < 0 || newParity > 2)
			throw new Exception("Invalid Serial Port Parity");
		parity = newParity;
		}
/**
 * Set the number of data bits
 * @exception Exception If number of data bits is out of range.
 */
	public void setDataBits(int newDataBits) throws Exception {
		if (newDataBits < 0 || newDataBits > 3)
			throw new Exception("Invalid Serial Port Data Bits");
		dataBits = newDataBits;
		}
/**
 * Set the number of stop bits
 * @exception Exception If number of stop bits is out of range.
 */
	public void setStopBits(int newStopBits) throws Exception {
		if (newStopBits < 0 || newStopBits > 1)
			throw new Exception("Invalid Serial Port Stop Bits");
		stopBits = newStopBits;
		}
/**
 * Set the handshake protocol
 * @exception Exception If handshake protocol is out of range.
 */
	public void setHandshake(int newHandshake) throws Exception {
		if (newHandshake < 0 || newHandshake > 15)
			throw new Exception("Invalid Serial Port Handshake");
		handShake = newHandshake;
		}
/**
 * Get the port number
 */
	public int getPort()      { return port+1; }
/**
 * Get the I/O address
 */
	public int getIoAdr()     { return ioAdr; }
/**
 * Get the interrupt request (IRQ) assignment
 */
	public int getIrq()       { return irq; }
/**
 * Get the transmit buffer length
 */
	public int getTxLen()     { return txLen; }
/**
 * Get the receive buffer length
 */
	public int getRxLen()     { return rxLen; }
/**
 * Get the bit rate
 */
	public int getBitRate()   { return bitRate; }
/**
 * Get the parity
 */
	public int getParity()    { return parity; }
/**
 * Get the number of data bits for each character
 */
	public int getDataBits()  { return dataBits; }
/**
 * Get the number of stop bits for each character
 */
	public int getStopBits()  { return stopBits; }
/**
 * Get the handshake protocol
 */
	public int getHandshake() { return handShake; }
/*
 * static data
 */
	public final static int HS_NONE = 4;
	public final static int HS_CTSRTS = 0;
	public final static int HS_DSRDTR = 10;
	public final static int HS_XONXOFF = 7;

	public final static int BR_110    = 0;
	public final static int BR_150    = 1;
	public final static int BR_300    = 2;
	public final static int BR_600    = 3;
	public final static int BR_1200   = 4;
	public final static int BR_2400   = 5;
	public final static int BR_4800   = 6;
	public final static int BR_9600   = 7;
	public final static int BR_19200  = 8;
	public final static int BR_38400  = 9;
	public final static int BR_57600  = 10;
	public final static int BR_115200 = 11;

	public final static int PY_NONE  = 0;
	public final static int PY_ODD   = 1;
	public final static int PY_EVEN  = 2;
	public final static int PY_MARK  = 3;
	public final static int PY_SPACE = 4;

	public final static int LN_5BITS = 0;
	public final static int LN_6BITS = 1;
	public final static int LN_7BITS = 2;
	public final static int LN_8BITS = 3;

	public final static int ST_1BITS = 0;
	public final static int ST_2BITS = 1;
/*
 * class data
 */
	int port=0; // port number (use COMn-1 for standard serial cards)
	int ioAdr=0x3F8; // N/A for multi-port cards
	int irq=4; // N/A for multi-port cards
	int txLen=2048;
	int rxLen=4096;
	int bitRate=BR_9600; // 9600bps
	int parity=PY_NONE; // none
	int dataBits=LN_8BITS; // 8 data bits
	int stopBits=ST_1BITS; // 1 data bit
	int handShake=HS_NONE; // none
//	int stdPort[] = { 0, 1 }

}