/*-----------------------------------------------------------------------------
 * File: SerialPort.java
 * Function: Class for interface to serial port(s)
 *
 * Requires jSerPort.DLL or jSerPortNm.DLL and associated DLL's
 *---------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
 * Solutions Consulting makes no representations or warranties about the
 * suitability of the software, either express or implied, including but
 * not limited to the implied warranties of merchantability, fitness for
 * a particular purpose, or non-infringement. Solutions Consulting shall
 * not be liable for any damages suffered by licensee as a result of
 * using, modifying or distributing this software or its derivatives.
 *---------------------------------------------------------------------------*/
//package SerialIO;

import java.io.*;

/*---------------------------------------------------------------------------*/
/**
 * Class for interface to serial port(s).
 * This class allows high-performance serial port throughput as well as
 * low-level serial port access to the serial port(s) of the computer.
 *
 * Since serial connections can be unreliable we choose not to use blocking
 * on the serial port. Instead an efficient timeout mechanism was implemented
 * which gives up its timeslice if no work is required. One could effectively
 * emmulate blocking by choosing a very large timeout.
 * <p><b>
 * <li>Copyright (c) 1996,1997 Solutions Consulting, All Rights Reserved.
 * </p></b>
 *
 * <li> loads proper DLL to support CNM/RNI/Win/OS/2
 * <li> Constructor uses SerialConfig object
 * @version 2.0 12 June 1997
 * @author David H. Boydston
 */
/*---------------------------------------------------------------------------*/
public class SerialPort {

// do this when object created...before constructor

	static {
		try {
			String jdkVer = new String(System.getProperty("java.version"));
			String vendor = new String(System.getProperty("java.vendor.url"));
			String osName = new String(System.getProperty("os.name"));
			String lib = "jSerPortNm";
			if (vendor.equals("http://www.microsoft.com/")) {
				lib = "jSerPortRni";
			}
			if (osName.equals("OS/2"))
				lib = "jspos2";
			if (osName.equals("Windows 3.1 (Win32s)"))
				lib = "jspw3";
			System.loadLibrary(lib);
			System.out.println("SerialPort class loaded: "+lib);
			}
		catch(UnsatisfiedLinkError ule){
			System.out.println(ule);
			System.exit(1);
			}
		}

/*
 * Constructor(s)
 */
/**
 * Create a SerialPort object based on a SerialConfig object
 * @param cfgObj The serial port configuration object
 */
	public SerialPort(SerialConfig cfgObj)
		throws IOException
		{
		int stat;
		String errMsg;

		stat = SerOpenPort(cfgObj);

		if (stat != 0) {
			if (stat == 7) // dialog for most probable cause
				errMsg = "SerOpenPort failed: Port is in use or not valid";
			else
				errMsg = "SerOpenPort failed: " + stat;
			throw new IOException(errMsg);
		}

		configure(cfgObj);
/*
 * we save a copy in this class to make our performance a bit faster.
 * Since we don't provide access to set this variable this is safe
 */
		port = cfgObj.getPort()-1; // make things a bit faster for this class

	} // end constructor

/*
 * class method(s)
 */
/**
 * Configures port with the parameters in the SerialConfig object
 * See documentation for details on the SerialConfig parameters.
 * @param cfgObj The serial port configuration object
 */
	public void configure(SerialConfig cfgObj)
		throws IOException
		{

		int stat = SerConfigure(cfgObj);

		if (stat != 0) {
			Integer si = new Integer(stat);
			throw new IOException("Configure failed: "+si.toString());
			}
		}
/**
 * Close a port object. The port should be closed
 * when its use is no longer required
 * @exception IOException if port not open (i.e. initialized)
 */
	public void close() throws IOException {
		int stat;

		if (port >= 0) {
			stat = SerClosePort(port);
			if (stat != 0)
				throw new IOException("ClosePort failed");
			port = -1; // mark invalid
		}
	}
/**
 * Send byte to the transmit buffer for transmission.
 * Returns after GetTimeout number of tics if insufficient room in TxBuf.
 * @param data The byte of data to output to the objects serial port
 * @exception IOException if port not open or transmit buffer full
 */
	public void putByte(byte data) throws IOException {

		int stat = SerPutByte(port, data);

		if (stat != 0) throw new IOException("PutByte failed");
	}
/**
 * Send String contents to the transmit buffer for transmission.
 * This will only pass UTF8 strings (i.e. 8 bit ASCII, not 16 bit UNICODE).
 * The string data is then transmitted.
 * @param s The string to output to the objects serial port.
 * @exception IOException if all characters in string are not sent
 * to the transmit buffer
 */
	public void putString(String s) throws IOException {

		int sLen = s.length();
		int stat = SerPutString(port, s);

		if (stat != -1) { // get -1 if all characters sent
			String msg = new String(
			"PutString: sent only "+stat+" of "+sLen+" bytes");
			throw new IOException(msg);
		}
	}
/**
 * Send byte array contents to the transmit buffer for transmission
 * Returns after GetTimeout number of tics if insufficient room in TxBuf.
 * @exception IOException if port not open or all bytes in array
 * @param data The byte array of data to output to the objects serial port
 * are not sent to the transmit buffer
 */
	public void putData(byte[] data) throws IOException {

		int dLen = data.length;
		int stat = SerPutData(port, data, dLen);

		if (stat != dLen) {
			String msg;
			if (stat == -1)
				msg = "PutData: port not initialized";
			else
				msg = new String(
					"PutData: sent only "+stat+" of "+dLen+" bytes");
			throw new IOException(msg);
		}
	}
/**
 * Send secificed length of byte array contents to the transmit buffer for
 * transmission Returns after GetTimeout number of tics if insufficient room
 * in TxBuf.
 * @param data The byte array of data to output to the objects serial port
 * @param dLen The length from the beginning of the byte array that will be
 * output to the objects serial port.
 * @exception IOException if port not open or all bytes in array
 * are not sent to the transmit buffer
 */
	public void putData(byte[] data, int dLen) throws IOException {

		int stat = SerPutData(port, data, dLen);

		if (stat != dLen) {
			String msg;
			if (stat == -1)
				msg = "PutData2: port not initialized";
			else
				msg = new String(
					"PutData2: sent only "+stat+" of "+dLen+" bytes");
			throw new IOException(msg);
		}
	}
/**
 * Get the next available byte from the receive buffer.
 * Returns after GetTimeout number of tics if byte not available.
 * @return byte or -1 if byte not available
 */
	public int getByte() {

		int data = SerGetByte(port);

		return data;
	}

/**
 * Get a copy of the next available byte from the receive buffer.
 * This is the byte that will be returned by the next call to GetByte()
 * @return byte or -1 if the port not open, or a byte is not available
 */
	public int peekByte() {

		int data = SerPeekByte(port);

		return data;
	}
/**
 * Transfers bytes into the data buffer from the serial port receive buffer.
 * Returns after GetTimeout number of tics if byte not available.
 * @param buf The byte array that will contain the data read from the objects
 * serial port. The routine uses buf.length determine the amount of data it
 * will attempt to read from the receive buffer.
 * @exception IOException if port not open
 * @return number of bytes transferred
 */
	public int getData(byte[] buf) throws IOException {

		int bufLen = buf.length; // don't
		int stat = SerGetData(port, buf, bufLen);

 		if (stat == -1) throw new IOException("GetData failed");

		return stat;
	}
/**
 * Returns the port number for this serial port object
 */
	public int getPortNum() {
		return port;
	}
/**
 * Returns the size of the receive buffer of serial port object
 * @exception IOException if port not open
 */
	public int rxLen() throws IOException {

		int size = SerRxLen(port);

		if (size == -1) throw new IOException("RxLen failed");

		return size;
	}
/**
 * Returns the size of the transmit buffer of serial port object
 * @exception IOException if port not open
 */
	public int txLen() throws IOException {

		int size = SerTxLen(port);

		if (size == -1) throw new IOException("TxLen failed");

		return size;
	}
/**
 * Returns the available space (in bytes) for the objects receive buffer
 * @exception IOException if port not open
 */
	public int rxAvail() throws IOException {

		int size = SerRxAvail(port);

		if (size == -1) throw new IOException("RxAvail failed");

		return size;
	}
/**
 * Returns the available space (in bytes) for the objects transmit buffer
 * @exception IOException if port not open
 */
	public int txAvail() throws IOException {

		int size = SerTxAvail(port);

		if (size == -1) throw new IOException("TxAvail failed");

		return size;
	}
/**
 * Flushes the objects receive buffer
 * @exception IOException if port not open
 */
	public int rxFlush() throws IOException {

		int stat = SerRxFlush(port);

		if (stat == -1) throw new IOException("RxFlush failed");

		return stat;
	}
/**
 * Flushes the objects transmit buffer
 * @exception IOException if port not open
 */
	public int txFlush() throws IOException {

		int stat = SerTxFlush(port);

		if (stat == -1) throw new IOException("TxFlush failed");

		return stat;
	}
/**
 * Returns true if the objects receive buffer is empty
 */
	public boolean rxEmpty() {

		int stat = SerRxEmpty(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns true if the objects transmit buffer is empty
 */
	public boolean txEmpty() {

		int stat = SerTxEmpty(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns true if the objects receive buffer overflowed
 * since the last call to this method.
 */
	public boolean rxOverflow() {

		int stat = SerRxOverflow(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns state of serial port DSR (Data Set Ready) signal
 */
	public boolean sigDSR() {

		int stat = SerSigDSR(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns state of serial port CTS (Clear To Send) signal
 */
	public boolean sigCTS() {

		int stat = SerSigCTS(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns state of serial port CD (Carrier Detect) signal
 */
	public boolean sigCD() {

		int stat = SerSigCD(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns true if the serial port for this object had a Framing Error
 * since last call to this method.
 */
	public boolean sigFrameErr() {

		int stat = SerSigFrameErr(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns true if the serial port UART for this object had a was Overrun
 * since last call to this method.
 */
	public boolean sigOverrun() {

		int stat = SerSigOverrun(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns true if the serial port for this object had a Parity Error
 * since last call to this method.
 */
	public boolean sigParityErr() {

		int stat = SerSigParityErr(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns true if the serial port for this object detected a Ring on the
 * line since the last call to this method.
 */
	public boolean sigRing() {

		int stat = SerSigRing(port);

		return (stat == 1) ? true: false;
	}
/**
 * Returns true if the serial port for this object detected a break on the
 * line since the last call to this method.
 */
	public boolean sigBreak() {

		int stat = SerSigBreak(port);

		return (stat == 1) ? true: false;
	}
/**
 * Set the DTR signal of objects serial port
 * @param onOff The state that the objects DTR line will be set to.
 * @exception IOException if port not open
 */
	public void setDTR(boolean onOff) throws IOException {

		int stat = SerSetDTR(port, onOff);

		if (stat == -1) throw new IOException("SetDTR failed");
	}
/**
 * Set the RTS signal of objects serial port
 * @param onOff The state that the objects RTS line will be set to.
 * @exception IOException if port not open
 */
	public void setRTS(boolean onOff) throws IOException {

		int stat = SerSetRTS(port, onOff);

		if (stat == -1) throw new IOException("SetRTS failed");
	}
/**
 * Send a break signal on the line
 * @param tics The number of milliseconds for the break signal. Note that this
 * value will be converted to the 'tic resolution' of the operating system.
 * The normal value for the tic on a Win95/WinNT X86 based PC is 54.95ms.
 * The length of the break will always be at least 1 tic.
 * @exception IOException if port not open
 */
	public void sendBreak(int tmo) throws IOException {

		int tics = tmo/54; // calculate timeout in tics (assume PC clock)
		if (tics == 0) tics = 1; // ensure one tic

		int stat = SerSendBreak(port, tics);

		if (stat == -1) throw new IOException("SendBreak failed");
	}
/**
 * Returns the timeout interval of serial port object.
  * @exception IOException if port not open
 */
	public int setTimeout() throws IOException {

		int interval = SerGetTimeout(port);

		if (interval == -1) throw new IOException("GetTimeout failed");

		return interval;
	}
/**
 * Sets the timeout of the serial port object.
 * The default timeout is zero meaning:
 * <li> Receive functions return immediately if byte(s) are not available.
 * <li> Transmit functions return immediately if TxBuf has insufficient room.
 * When the timeout is set to X, receive functions will wait for X amount of
 * time for the completion of the function, before returning, and transmit
 * functions will wait X amount of time for the TxBuf to drain.
 * @param tics The number of milliseconds for the timeout. Note that this
 * value will be converted to the 'tic resolution' of the operating system.
 * The normal value for the tic on a Win95/WinNT X86 based PC is 54.95ms.
 * @exception IOException if port not open
 */
	public void setTimeout(int tmo) throws IOException {

		int tics = tmo/54; // calculate timeout in tics (assume PC clock)
		if (tmo > 0 && tmo < 54) tics = 1; // ensure one tic for small vals

		int stat = SerSetTimeout(port, tics);

		if (stat == -1) throw new IOException("SetTimeout failed");
	}
/**
 * Returns the version number of the current serial port DLL
 * @return
 * <li> The most significant 16 bits contain the major version
 * <li> The least significant 16 bits contain the minor version
 */
	public int getLibVer() {
		return SerGetLibVer();
	}
/**
 * Be sure that port is cleaned up before exit (in case close was not called)
 */
	protected void finalize() {
		try {
			this.close();
			} catch (IOException ioe) {}
	}

/*
 * class data
 */
	int port=-1;
/*
 * native methods
 * note: direct calls to these native methods is NOT supported
 */
	native int SerGetLibVer();
	native int SerOpenPort(SerialConfig cfg);
	native int SerConfigure(SerialConfig cfg);
	native int SerClosePort(int port);
	native int SerPutByte(int port, byte b);
	native int SerPutString(int port, String s);
	native int SerPutData(int port, byte[] data, int cnt);
	native int SerGetByte(int port);
	native int SerGetData(int port, byte[] data, int cnt);
	native int SerPeekByte(int port);
	native int SerSetTxPace(int port, int pTime);
	native int SerGetTxPace(int port);
	native int SerRxLen(int port);
	native int SerTxLen(int port);
	native int SerRxAvail(int port);
	native int SerTxAvail(int port);
	native int SerRxFlush(int port);
	native int SerTxFlush(int port);
	native int SerRxEmpty(int port);
	native int SerTxEmpty(int port);
	native int SerRxOverflow(int port);
	native int SerSetDTR(int port, boolean onOff);
	native int SerSetRTS(int port, boolean onOff);
	native int SerSendBreak(int port, int tics);
	native int SerGetTimeout(int port);
	native int SerSetTimeout(int port, int tics);
	native int SerSigCTS(int port);
	native int SerSigDSR(int port);
	native int SerSigCD(int port);
	native int SerSigFrameErr(int port);
 	native int SerSigParityErr(int port);
 	native int SerSigOverrun(int port);
	native int SerSigRing(int port);
	native int SerSigBreak(int port);
}